############################################################
package Presentator::Format;
############################################################
use strict;
use Unicode::MapUTF8 qw(to_utf8); # Prozatim provedu mapovani primo zde, dale by melo byt deklarovano nekde lepe.
use vars qw/$VERSION/;

$VERSION = "0.01";

1;

sub new {
	my $class = shift;
	$class = ref $class || $class;
	my %attr = @_;
	my $self = {
		format => [],
		query => undef, # query is created at processing stage
		output => $attr{"output"},
		fieldTags => {} 
	};
	bless $self, $class;
	return $self;
}	

sub getQuery {
	my $self=shift;
	return $self->{"query"};
}

sub getFormat {
	my $self = shift;
	return @{$self->{"format"}};
}

sub addFormat {
	my $self = shift;
	my $data = shift;
	push (@{$self->{"format"}}, $data);
}

sub addRef {
	my $self = shift;
	my $value = shift;
	$self->addFormat({type => "Ref", value => $value});
}

sub addMarkup {
	my ($self, $sub) = @_;
	$self->addFormat({type => "Markup", value => $sub});
}

sub addData {
	my $self = shift;
	my $name = shift;
	$self->addFormat({type => "Data", value => $name});
	$self->{fieldTags}{$name}=$name;
}

sub process {
	my ($self,%attr) = @_;
	foreach ($self->getFormat) {
		if ($_->{"type"} eq "Ref") {
			$_->{"value"}->processIter(pre_results=>[]);
		} elsif ($_->{"type"} eq "Text") {
			$self->append($_->{"value"}, "UTF-8");
		} elsif ($_->{"type"} eq "Markup") {
			&{$_->{"value"}}();
		}
	}
	$self->eraseQueries();
}

sub eraseQueries {
	my $self=shift;
	map {$_->{"value"}->eraseQueries() if ($_->{"type"} eq "Ref"); } 
		($self->getFormat);
	$self->{query} = undef;
}

## Param 1 (\@) ... list of previous queries
## Param 2 (\@) ... list of previous results

sub processIter {
	my ($self,%attr) = @_;

	my $query = $self->getQuery;
	
	my $result = $query->select($attr{pre_results});
	return unless $result;
	while ($result->nextRow) {
		foreach ($self->getFormat) {
			if ($_->{"type"} eq "Ref") {
				push(@{$attr{pre_results}}, $result);
				$_->{"value"}->processIter(%attr);
				pop(@{$attr{pre_results}});
			} elsif ($_->{"type"} eq "Data") {
				my $field = $result->field($_->{"value"});
				$self->{"output"}->append($field->value, $field->encoding);
			} elsif ($_->{"type"} eq "Markup") {
				&{$_->{"value"}}();
			}
		}
	}
}

__END__

=head1 NAME

Presentator::Format - class used for processing of iterative parts

=head1 DESCRIPTION

This module is used for processing of C<form> and C<iterate> parts of template.

=head1 INTERFACE

=over 4

=item addMarkup($sub)

This function adds the parts of iterate that are not processed, i.e. characters, processing_instructions and start/end_tags. These are copied into the result when iteration is processed. The only parameter C<$sub> is the reference to the function that takes no parameters and that performs actual output.

=item addRef($subformat)

This function adds the reference to the iteration part. These parts are processed recursively.

=item addData($name, \%attributes)

This function adds reference to the retrieved data. During the processing these references are substituted with actual retrieved data.

=item process()

This function is used for actual processing of requirements. The queries are performed and C<form> and C<iterate> parts are recursively applied to the retrieved data.

=back

=head1 SEE ALSO

L<Presentator>, L<Presentator::Intro>, L<Presentator::Select>

=head1 AUTHOR

Copyright (c) 2002 Igor Chladil (ichladil@quick.cz). All rights reserved.

This program is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
