############################################################
package Presentator::PresentatorSAX;
############################################################
use strict;
use vars ('@ISA');
use XML::SAX::Base;
@ISA = ('XML::SAX::Base');
use Presentator::Format;

use Unicode::MapUTF8 qw(to_utf8); # Provadim prevod do znakove soustavy UTF-8

1;

sub new {
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $self = bless $proto->SUPER::new(@_), $class;
	my %mine = (
		PRE_URI => "http://www.pef.mendelu.cz/fis/presentator",
		iter_no => 0,
		iter_stack => [], 
		in_form => 0,
		format => [],
		query_stack => [],
		query_no => 0,
		query_where => undef,
		dbh => undef,
		document => '',
		queries_params => undef,

		inputenc => "UTF-8",
		outputenc => "UTF-8",

		debug => 0
	);
	
	while (my($k,$v) = each %mine) {
		$self->{$k} = $v;
	}
	
	my %attr = @_;
	$self->{"queries_factory"} = $attr{"queriesFactory"};
	$self->{"PRE_URI"} = $attr{"URI"} if exists $attr{URI};
	return $self;
}

sub iterStackTop {
	my $self = shift;
	my $ret = $self->{"iter_stack"}[@{$self->{"iter_stack"}}-1];
	return $ret;
}

sub curQuery {
	my $self = shift;
	if (@{$self->{"query_stack"}}) {
		return $self->{"format"}[$self->{"query_no"}]->getQuery;
	} else {
		return undef;
	}
}

sub debug {
	return $_[0]->{"debug"};
}

sub getDocument {
	my $self = shift;
	return $self->{"document"};
}

sub append {
	my $self = shift;
	my $string = shift; 
	my $inputenc = shift || $self->{"inputenc"}; 
	$self->SUPER::characters({Data =>
		($inputenc ne $self->{"outputenc"})?
		(to_utf8({-string => $string, -charset => $inputenc})):
		($string)}
	);
}

sub addMarkup {
	my ($self, $sub) = @_;
	$self->{"format"}->[$self->iterStackTop]->addMarkup($sub);
}

sub characters {
	my ($self, $data) = @_;
	if ($self->{"in_form"}) {	
		$self->addMarkup(sub {$self->SUPER::characters($data);});
	} elsif (!$self->{"query_no"}) {
		$self->SUPER::characters($data);
	}
}

sub start_element {
	my ($pre, $data) = @_;
	my $elem = $data->{LocalName};
	my $ns = ($data->{NamespaceURI} or "");
	my %sax_attr = ();
	%sax_attr = %{$data->{Attributes}} if $data->{Attributes};

	if ($ns eq $pre->{"PRE_URI"}) {
		## Upravim atributy do puvodniho formatu
		my %attr;
		foreach my $att (values %sax_attr) {
			if ((!$att->{NamespaceURI})
				or($att->{NamespaceURI} eq $pre->{PRE_URI})) {
				$attr{$att->{LocalName}} = $att->{Value};
			}
		}
		
		if ($elem =~ /^(form|iterate|query)$/) { # Processing statements
			if ($elem eq "form") { #Vstupuji do formy
				$pre->{"in_form"}=1;
				$pre->{"iter_no"} = 0;
					$pre->{"iter_stack"} =[0];
				$pre->{"format"} = [new Presentator::Format(output => $pre)];
			}
			if ($elem eq "iterate") { #Vstupuji do dalsi iterace 
				$pre->{"iter_no"}++;
				push (@{$pre->{"iter_stack"}}, $pre->{"iter_no"});
				push (@{$pre->{"format"}}, Presentator::Format->new(dbh => $pre->{"dbh"}, 
					output => $pre));
			}

			############# Zpracovani dotazu ################
			if ($elem eq "query") { 
				my $par_name;
				$attr{"name"} =~ /^(.*:)?(.*)$/io;
				$attr{"name"} = $2 || $1;
				my $query = $pre->{"queries_factory"}->createSelect($1);
				## Vstupuji do poddotazu
				if (@{$pre->{"query_stack"}}) {
					$pre->curQuery->addHasSubquery([$attr{"name"}], $attr{"as"});
					$par_name = $pre->curQuery->getQueryName;
				}
				
				$pre->{"query_no"}++;
				$query->addFieldTags(keys %{$pre->{format}[$pre->{query_no}]{fieldTags}});
				$pre->{"format"}[$pre->{"query_no"}]{"query"} = $query;
				push(@{$pre->{"query_stack"}}, $pre->{"query_no"});
				$pre->curQuery->setQueryName($attr{"name"});

				## Pozor, jsem v poddotazu
				if (scalar @{$pre->{"query_stack"}}>1) { 
					$pre->curQuery->addIsSubquery([$par_name], $attr{"as"});
				}
			}
			################ Konec zpracovani dotazu ###################
			
		} elsif($pre->{"in_form"}) {
				# Pridam do aktualnich $query_data soucasny element
				$pre->{"format"}[$pre->iterStackTop]->addData($elem); 
				print ("ADDING FIELD: $elem\n") if $pre->{"debug"}>1; 
		} elsif ($pre->{"query_where"}) {
			$pre->curQuery->addWhereTag($elem, %attr);
		} elsif ($pre->{"query_no"}) {
			if ($elem eq "where") {
				$pre->{"query_where"}="1";	
				$pre->curQuery->addWhereTag('and'); # AND is implicit conjunction
			} elsif ($elem eq "order") { ### Pridam poradi
				$pre->curQuery->addOrderBy(\%attr);
			}
		}
	} else {
		if ($pre->{"in_form"}) {
			$pre->addMarkup(sub {$pre->SUPER::start_element($data)});
		} else {
			$pre->SUPER::start_element($data);
		}
	}
}

sub end_element {
	my ($pre, $data) = @_;
	my $elem = $data->{LocalName};
	my $ns = ($data->{NamespaceURI} or "");
	if ($ns eq $pre->{"PRE_URI"}) {
		# That is the condition
		if (($elem=~ /^(and|or|not)$/io) and $pre->{"query_where"}) {
			$pre->curQuery->addWhereTag($elem."_end");
		}
		if ($elem eq "form") { #Vystupuji do formy
			$pre->{"in_form"}=0;
			#$pre->{"format"}[0]->printDebug;
		}
		if ($elem eq "iterate") { #Vystupuji do dalsi iterace 
			my $ref=pop(@{$pre->{"iter_stack"}});
			$pre->{"format"}[$pre->iterStackTop]->addRef($pre->{"format"}[$ref]);
		}
		if ($elem eq "query") { #Vystupuji do dotazu
			my $curquery = $pre->{"format"}[pop(@{$pre->{"query_stack"}})]->getQuery;
#			$curquery->processFieldTags;
			if(!@{$pre->{"query_stack"}}) {
			## Zde se zpracuje cela sablona a vsechny poddotazy
				$pre->{"format"}[0]->process(dbh=>$pre->{dbh},output=>$pre); 
				$pre->{"query_no"}=0;
			}
		}
		if ($elem eq "where") {
			$pre->curQuery->addWhereTag("and_end"); #and of implicit logical conjunction
			$pre->{"query_where"} = undef;
		}
	} else {
		if ($pre->{"in_form"}) {
			$pre->addMarkup(sub {$pre->SUPER::end_element($data)});
		} else {
			$pre->SUPER::end_element($data);
		}
	}
}

