package Presentator::Queries;

use strict;
use Carp;
use vars qw/@ISA/;
@ISA = qw/Presentator::AbstractParts/;

sub new {
	my $proto = shift;
	my $class = ref $proto || $proto;
	my $self = $class->SUPER::new(@_);
	bless $self, $class; #reincarnation
#	my $context = $self->xp->find('/queries');

#	unless ($context->size()>0 || $self->context) {
#		warn "NO Context found."; return
#	}

	#The default context is set
#	$self->{context} = $context->get_node(1) 
#		unless exists $self->{context};

	$self->{encoding} = 'UTF-8' unless $self->encoding;
	$self->{cndlist} = {};
	
	#and now all the queries are initialised
	my @queryNodes = $self->findnodes('./query');
	foreach my $QueryContext (@queryNodes) {
		my $query = Presentator::Query->new(
			xpath=>$self->{xpath},
			encoding=>$self->{encoding},
			context=>$QueryContext,
			access_order=>$self->{access_order},
			access_allow=>$self->{access_allow},
			access_deny=>$self->{access_deny},
			cndlist => $self->{cndlist});
		$self->{queries}{$query->name()} = $query;
	}

	#and now the subquery links should be initialised

	foreach my $query (values %{$self->{queries}}) {
		my @isSQ = $query->findnodes('./subquery');
		foreach my $SQ (@isSQ) {
			my $from = $SQ->findvalue('./@from');
			my $role = $SQ->findvalue('./@as');
			my @sources = map {$self->query($_)} split(/ /, $from);
			my $cnd = new Presentator::Condition(xpath=>$self->xp,context=>$SQ,
				cndlist=>$self->{cndlist});
			$query->_addIsSubquery($role, \@sources, $cnd);
			map {$_->_addHasSubquery($query, $role, \@sources, $cnd)} @sources;
		}
	}

	#finally the reference is blessed
	return bless $self, $class;
}

sub queries {
	return %{$_[0]->{queries}} if not $_[1];
	my %res;
	while (my ($name,$field) = each %{$_[0]->{queries}}) {
		$res{$name}=$field if $field->isAllowed($_[1]);
	}
	return %res;
}

sub query {
	return $_[0]->{queries}{$_[1]}->isAllowed($_[2])?
		$_[0]->{queries}{$_[1]}:undef;
}

sub writeXML {
	my $self = shift;
	my %args = @_;
	return unless exists $args{OUTPUT};
	my $out = $args{OUTPUT};
	$out->startTag("queries",  
		"access"=>($self->isAllowed($args{access_groups})?"allow":"deny") );
	$out->dataElement("doc", $self->doc());
	if ($args{RECURSE}) {
		map {$_->writeXML(@_)} values %{$self->{queries}};
	} else {
		map {
			$out->startTag("query", "name" => $_->name(), 
			"access"=>($self->isAllowed($args{access_groups})?"allow":"deny") );
			$out->dataElement("doc", $_->doc());
			$out->endTag;
			}
			values %{$self->{queries}};
	}
	$out->endTag();
}

package Presentator::AbstractQueryPart;
use Carp;
use vars qw/@ISA/;

@ISA = qw/Presentator::AbstractParts/;

sub new {
	my $proto = shift;
	my $class = ref $proto || $proto;
	my $self = $class->SUPER::new(@_);
	bless $self, $class; #reincarnation
	
	# Now processing the conditions via cnds or without it
	my $cndRefs = $self->findvalue('@cnds');
	my @cndRefsAr = split /\s+/,$cndRefs; # Rada s odkazy na podminky

	my $names = join(' or ', map("\@name=\"$_\"", @cndRefsAr));
	my @cnds;
	if ($names) {
		@cnds = $self->findnodes('./ancestor-or-self::query/cnds['.$names.']/addwhere|./addwhere');	
	} else {
		@cnds = $self->findnodes('./addwhere');	
	}

	# Now the addwhere tags should be processed and the required parameters must be find. These conditions will be processed only once for each query even though they may contain parameters (these will be bound only once as well).
	@cnds = map(new Presentator::Condition(xpath=>$self->xp, context=>$_,
		cndlist=>$self->{cndlist}),
		@cnds);
	$self->{addwhere} = \@cnds;

	my @froms = ();
	if ($names) {
		@froms = $self->findnodes('./ancestor-or-self::query/cnds['.$names.']/addfrom/text()|./addfrom/text()')
	} else {
		@froms = $self->findnodes('./addfrom/text()');
	}
	
	@froms = map {$_->getData()} @froms;
	$self->{addfrom} = \@froms;

	return $self;
}

sub addfrom {
	return @{$_[0]->{addfrom}};
}

sub addwhere {
	return @{$_[0]->{addwhere}};
}


1;

package Presentator::AbstractParts;
use Carp;

sub new {
	my @options = qw(xpath context encoding access_allow access_deny access_order cndlist);
	#cndlist is used in Condition for referencing
	my $proto = shift;
  my $class = ref $proto || $proto;
	my %args = @_;
	my %self = map((exists $args{$_})?($_ => $args{$_}):(), @options);
	my $self = \%self;
	
	bless $self, $class;
	
	# Return undef on no XPath object found
	unless (exists $self{xpath}) {
		carp "No XPath object found"; return
	}
	
	# Initialisation of default context
	unless (exists $self{context}) {
		my $context = $self->{xpath};
		$self{context}=$context;
		#default context is initialised
		#my $context = $self->find('/queries');
		#if ($context->size()>0) {
	#		$self{context}=$context->get_node(1)
	#	} else {
	#		carp "No context defined";
	#	}
	}
	
	# Initialisation of encoding
	my $encoding = $self->findvalue('@encoding');
	$self->{encoding}=$encoding if $encoding;

	# Initialisation of documentation
	$self{doc} = $self->findvalue('doc');

	# Get name of the query
	$self{name} = $self->findvalue('@name');

	# Now process the access policy and initialise access_order, access_allow and access_deny
	my @attrs = $self->context()->getAttributes();
	foreach my $att (@attrs) {
		if ($att->getName() eq 'allow') {
			$self{access_allow}={};
			map {$self{access_allow}->{$_}=1} split(/\s+/,$att->getValue())
		}
		if ($att->getName() eq 'deny') {
			$self{access_deny}={};
			map {$self{access_deny}->{$_}=1} split(/\s+/,$att->getValue())
		}
		if ($att->getName() eq 'order') {
			my $policy = $att->getValue();
			$policy =~ s/(^\s+)|(\s+$)//;
			$policy =~ s/\s+/,/;
			$self{access_order} = $policy;
		}
	}
	return $self;
}

sub xp {
	return $_[0]->{context}
}

sub find {
	my ($self, $what) = @_;
	$self->xp->find($what);
}

sub findvalue {
	my ($self, $what) = @_;
	$self->xp->findvalue($what);
}

sub findnodes {
	my ($self, $what) = @_;
	return ($self->xp->findnodes($what));
}

sub context {
	return $_[0]->{context}
}

sub doc {
	return $_[0]->{doc}
}

sub name {
	return $_[0]->{name}
}

sub encoding {
	return $_[0]->{encoding}
}

#isAllowed($groups_arref)
#This method should be passed in order to have access to this object.

sub isAllowed {
	my ($self, $groups) = @_;
	# return is allowed when there are no groups specified
	return 1 unless $groups;
	my ($allow, $deny) = (0,0);
	# testing for non-empty intersection of groups with allow|deny 
	foreach (@$groups) {
		$allow=1 if exists $self->{access_allow}{$_};
		$deny=1 if exists $self->{access_deny}{$_};
	}
#	print ("Allow = $allow, Deny = $deny, Order=$self->{access_order}\n");
	return ($allow and (not $deny)) if ($self->{access_order} eq 'allow,deny');
	return $allow if ($self->{access_order} eq 'allow');
	return ((not $deny) or $allow)  if ($self->{access_order} eq 'deny,allow');
	return (not $deny) if ($self->{access_order} eq 'deny');
	return 1; # return one if no security is specified
}


1; 

package Presentator::Query;
use Carp;
use vars qw(@ISA);
@ISA = qw(Presentator::AbstractQueryPart);

sub new {
	my $proto = shift;
  my $class = ref $proto || $proto;
	my $self = $class->SUPER::new(@_);

	# initialisation of possible fields
	$self->{fields} = {};
	my @fields = $self->findnodes('out');

	foreach my $fieldContext (@fields) {
		my $field = Presentator::Field->new(
			xpath=>$self->xp(),
			encoding=>$self->encoding(),
			context=>$fieldContext,
			access_allow=>$self->{access_allow},
			access_deny=>$self->{access_deny},
			access_order=>$self->{access_order},
			cndlist => $self->{cndlist},
		);
		$self->{fields}{$field->name()}=$field;
	}

	# loading group by clauses

	my @group_by = $self->findnodes('addgroupby/@out');

	my $group_by = join(" ",map($_->getData, @group_by));

	$self->{group_by} = undef;
	$self->{group_by} = [split(/\s+/, $group_by)]
		if scalar @group_by;
	

	# loading the conditions (where)

	my @conditions = $self->findnodes('where');
	
	$self->{conditions}={};

	foreach my $conditionContext (@conditions) {
		my $condition = Presentator::Condition->new(
			xpath=>$self->xp(),
			context=>$conditionContext,
			access_allow=>$self->{access_allow},
			access_deny=>$self->{access_deny},
			access_order=>$self->{access_order},
			cndlist => $self->{cndlist},
			);
		$self->{conditions}{$condition->name()} = $condition;
	}
	
	$self->{_issubquery} = [];
	$self->{_hassubquery} = [];

	return $self;
}

sub encoding {
	return $_[0]->{encoding}
}

sub group_by {
	return $_[0]->{group_by}
}

sub fields {
	return %{$_[0]->{fields}} if !$_[1];
	my %res;
	while (my ($name,$field) = each %{$_[0]->{fields}}) {
		$res{$name}=$field if $field->isAllowed($_[1]);
	}
	return %res;
}

sub field {
	return $_[0]->{fields}{$_[1]}->isAllowed($_[2])?
		$_[0]->{fields}{$_[1]}:undef;
}

sub conditions {
	return %{$_[0]->{conditions}} if !$_[1];
	my %res;
	while (my ($name,$field) = each %{$_[0]->{conditions}}) {
		$res{$name}=$field if $field->isAllowed($_[1]);
	}
	return %res;
}

sub condition {
	my $cnd = $_[0]->{conditions}{$_[1]};
	return ($cnd and $cnd->isAllowed($_[2]))?
		$_[0]->{conditions}{$_[1]}:undef;
}

sub _addIsSubquery {
	my ($self, $role, $sources, $cnd) = @_;
	push @{$self->{_issubquery}}, {role => $role, sources => $sources, condition=>$cnd};
}


sub _addHasSubquery {
	my ($self, $subquery, $role, $sources, $cnd) = @_;
	push @{$self->{_hassubquery}}, {role=>$role, subquery=>$subquery, sources=>$sources, condition=>$cnd};
}

sub hasSubquery {
	my $self =shift;
	return @{$self->{_hassubquery}};
}

sub isSubquery {
	my $self=shift;
	return @{$self->{_issubquery}};
}

sub debug {
	my $self = shift;
	my %fields = $self->fields();
	my %conditions = $self->conditions();
	print 
	"\nQUERY: NAME => ", $self->name(),
	"\tENCODING => ", $self->encoding(),
	"\tDOCUMENTATION => ", $self->doc();
	print join(',',map("\n\tHAS SUBQUERY: Role =>".$_->{role}."\tTarget => "
		.$_->{subquery}->name(),$self->hasSubquery));
	print join(',',map("\n\tIS SUBQUERY: Role =>".$_->{role}."\tFrom => "
		.join(',',@{$_->{sources}}),$self->isSubquery));
	map {print "\t";$_->debug()} values %fields;
	map {print "\t";$_->debug()} values %conditions;
	print "\n";
}

sub writeXML {
	my $self = shift;
	my %args = @_;
	return unless exists $args{OUTPUT};
	my $out = $args{OUTPUT};
	my %implicitCnds = (
		'gt' => 'greater than',
		'lt' => 'less than',
		'eq' => 'equal',
		'ge' => 'greater or equal',
		'le' => 'less or equal',
		'like' => 'like',
		'in' => 'in');
	$out->startTag("query", 
		"name" => $self->name(),
		"access"=>($self->isAllowed($args{access_groups})?"allow":"deny") );
	$out->dataElement("doc", $self->doc());
	if ($args{RECURSE}) {
		map {$_->writeXML(@_)} values %{$self->{fields}};
		map {$_->writeXML(@_)} values %{$self->{conditions}};
		my @implicitTypes = map("$_", keys %{$self->{fields}});
		foreach my $cnd (keys %implicitCnds) {
			$out->startTag("where", name=>$cnd, access=>"allow");
			$out->dataElement("doc", $implicitCnds{$cnd});
			$out->startTag("param",name=>"name", "type"=>'options:',
				"classes"=>'template');
			foreach my $type (@implicitTypes) {
				$out->emptyTag("option", name=>$type, value=>$type);
			}
			$out->endTag();
			$out->dataElement("param",'', name=>"value", "type"=>'variant',
				"classes"=>'template');
			$out->endTag;
		}
	} else {
		map {$out->dataElement("out", $_->doc(), "name" => $_->name(), 
		"encoding" => $_->encoding(), "type" => $_->type(),
		"access"=>($_->isAllowed($args{access_groups})?"allow":"deny") );
		}	values %{$self->{fields}};
		map {$out->dataElement("where", $_->doc(), "name" => $_->name(),
		"access"=>($_->isAllowed($args{access_groups})?"allow":"deny") );
		}	values %{$self->{conditions}};
	}
	$out->endTag();
}

##########################################################
######		Presentator::Field package                    ##
##########################################################

package Presentator::Field;
use Carp;
use vars qw(@ISA);
@ISA = qw(Presentator::AbstractQueryPart);

sub new {
	my $proto = shift;
	my $class = ref $proto || $proto;
	my $self = bless $class->SUPER::new(@_), $class;
	$self->{type} = $self->findvalue('@type');
	
	my @params = $self->findnodes('./param');
	$self->{params}=[];
	foreach my $paramsContext (@params) {
		push @{$self->{params}}, new Presentator::CndParam(
			context => $paramsContext,
			xpath => $self->xp,
		);
	}
	
	$self->{sql} = "";
	my @tpStrings = $self->findnodes('text()|param');
	foreach my $tpString (@tpStrings) {
		my $pom = '';
		if ($tpString->isa("XML::LibXML::Text")) {
			$pom=$tpString->getData();
		} else {
			$pom = '? ';
		}
		$self->{sql} .= $pom;
	}
	$self->{sql} =~ s/(\s)+/ /sgio; # Here the whitespace is chopped

	return $self;
}

sub encoding {
	return $_[0]->{encoding}
}

sub type {
	return $_[0]->{type}
}

sub params {
	return @{$_[0]->{params}}
}

sub sql {
	return $_[0]->{sql}
}

sub debug {
	my $self = shift;
	
	print 
	"FIELD: Name => ", $self->name(),
	"\tEnc => ", $self->encoding(),
	"\tDoc => ", $self->doc(), "\tType => ", $self->type(),"\n";
}

sub writeXML {
	my $self = shift;
	my %args = @_;
	return unless exists $args{OUTPUT};
	my $out = $args{OUTPUT};
	$out->startTag("out",
		"name" => $self->name(),
		"encoding" => $self->encoding(), "type" => $self->type(),
		"access"=>($self->isAllowed($args{access_groups})?"allow":"deny") );
	$out->dataElement("doc", $self->doc());

#	my @params = $self->params();
#	foreach (@params) {
#		$_->writeXML(OUTPUT => $out);
#	}
	$out->endTag();
}

##########################################################
######		Presentator::Condition package                ##
##########################################################
package Presentator::Condition;
use Carp;
use vars qw(@ISA);
@ISA = qw(Presentator::AbstractQueryPart);

sub _returnCnd {
	my ($xp, $context, $conditions) = @_;
	foreach (values %$conditions) {
		if ($context->isEqual($_->context)) {
			return $conditions->{$context};
		}
	}
	return undef;
}

sub new {
	my $proto = shift;
	my $class = ref $proto || $proto;
	my %args = @_;

	my $self = _returnCnd($args{xpath},$args{context}, $args{cndlist}); # It examines whether it wasn't defined earlier
	return $self if $self; # returns the existing condition if it was defined
	$self = $class->SUPER::new(@_); # otherwise new condition is created
	
	# Initialisation of parameters into hash with following elements:
	# name, default value, type (for type checking), context
	my @params = $self->findnodes('./param');
	$self->{params}=[];
	foreach my $paramsContext (@params) {
		push @{$self->{params}}, new Presentator::CndParam(
			context => $paramsContext,
			xpath => $self->xp,
		);
	}

	# Initialise the placeholding string
	$self->{placeholdingString} = '(';
	my @tpStrings = $self->findnodes('text()|param');
	foreach my $tpString (@tpStrings) {
		my $pom = '';
		if ($tpString->isa("XML::LibXML::Text")) {
			$pom=$tpString->getData();
		} else {
			$pom = '? ';
		}
		$self->{placeholdingString} .= $pom;
	}
	$self->{placeholdingString} =~ s/(\s)+/ /sgio; # Here the whitespace is chopped
	$self->{placeholdingString} .= ')';
	bless $self, $class;
	$self->{cndlist}->{$self->context} = $self; # and added into condition list
}

sub params {
	return @{$_[0]->{params}}
}

sub placeholdingString {
	return $_[0]->{placeholdingString};		
}

sub debug {
	my $self = shift;
	my @params = $self->params();
	
	print 
	"CONDITION: NAME => ", $self->name(),
	"\tDOCUMENTATION =>  ", $self->doc(), 
	"\n\tPlaceholding String => ", $self->placeholdingString(),
	"\tParameters => ", join(',', map ("$_->{name}($_->{default},$_->{type})", @params)),
	"\n";
}

sub writeXML {
	my ($self, %args) = @_;
	return unless exists $args{OUTPUT};
	my $out = $args{OUTPUT};
	my @params = $self->params();
	$out->startTag("where", "name" => $self->name(),
		"access"=>($self->isAllowed($args{access_groups})?"allow":"deny") );
	$out->dataElement("doc", $self->doc());
	foreach (@params) {
		$_->writeXML(OUTPUT => $out);
	}
	$out->endTag();
}

1;

package Presentator::CndParam;

use Carp;
use vars qw(@ISA);
@ISA = qw(Presentator::AbstractQueryPart);
use strict;

1;

sub new {
	my $proto = shift;
	my $class = ref $proto || $proto;
	my $self = bless $class->SUPER::new(@_), $class;
	$self->{type} = $self->findvalue('@type');
	$self->{default} = $self->findvalue('@default');
	
	my $classes = $self->findnodes('@classes');
	$self->{classes} = undef;
	$self->{classes} = [split(/\s+/,$classes)] if $classes->size();
	
	return $self;
}

sub type {
	return $_[0]->{type}
}

sub classes {
	return $_[0]->{classes}
}

sub default {
	return $_[0]->{default}
}

sub writeXML {
	my $self = shift;
	my %args = @_;
	return unless exists $args{OUTPUT};
	my $out = $args{OUTPUT};
	my %attr = (
		"name" => $self->name(),
		"type" => $self->type(),
		"default" => $self->default(),
	);
	$attr{"classes"} = join(' ', @{$self->classes()}) if $self->classes();
	$out->startTag("param", %attr);
	$out->dataElement("doc", $self->doc());
	$out->endTag();
}

=head1 NAME

Presentator::Queries - this package represents the top level element of data definition for Presentator.

Presentator::Query, Presentator::Condition, Presentator::Field, Presentator::CndParam

=head1 DESCRIPTION

The Presentator::Queries package interface serves as the entry point into the description of queries structure. All these packages should together provide unified means for description of available queries, their fields and conditions.

=head1 Implementation Details

=over 4

=item new()

The constructor new may take four parameters. The parameters are passed in hash form. The only required parameter is the XML::LibXML::Node object (-xpath). The parameter -encoding may define the default characters encoding for the whole tree. Default value is the value of @encoding attribute in Context element or UTF-8 if it is not defined. Next parameter may represent documentation of data definition. The documentation may be passed in -doc parameter. It is initialised to 'doc' child of the Context element by default.  

=back

=head1 INTERFACES

All above-mentioned interfaces will be described in following sections. All these interfaces have some common features (all are descendants of one interface) and this common interface will be described first. 

=head2 Common Features

=over 4

=item doc()

Returns the documentation of respective part. Documentation is not constraint by any means.

=item name()

Returns the name of respective part. It may be the name of the query, of the field, condition or parameter.

=item isAllowed(\@userGroups)

This function returns true if current user may access respective object. C<\@userGroups> is the array of user groups which current user belongs to.

=item writeXML(OUTPUT=E<gt>$out)

Parameters are passed as hash with possible keys C<OUTPUT>, C<RECURSE>, C<access_groups>.
C<OUTPUT> is the only required parameter. Standard value for RECURSE is C<true>.

Write XML provides unified interface for serialization of objects defined here. C<OUTPUT> is XML::Writer object. C<RECURSE> defines whether the output should be deep (recursive) or not. C<access_groups> are user groups passed to the C<isAllowed>; these are passed as array reference. 

Further description is in commented DTD F<writexml.dtd>.

=back

=head2 Presentator::Queries

This object is the entry point to the description of the C<query> objects. 

=over 4

=item queries()

Returns the array of all C<Presentator::Query> objects defined within the C<Presentator::Queries>.

=item query($name)

Returns C<Presentator::Query> object with name C<$name>.

=back

=head2 Presentator::Query

This interface provides means for accessing informations about C<Query> objects.

=over 4

=item fields(),conditions()

These functions returns array of all C<Presentator::Field> or C<Presentator::Condition> defined within current C<query>.

=item field($name),condition($name)

These functions returns C<Presentator::Field>, resp. C<Presentator::Condition> object with the name C<$name>.

=item hasSubquery()

Returns the array of hash references. Hash references contain following fields:

Value of C<subquery> represents C<Presentator::Query> that may serve as the subquery. 

Value of C<role> defines the relation of the subquery to the current query.

Value of C<sources> is the array of C<Presentator::Query> objects. These objects serves as the parents for the subquery. It may be used for n-ary relations.

=item isSubquery()

Returns the array of hash references. Hash references contain following fields:

Value of C<role> defines the role of current query.

Value of C<sources> is the array of C<Presentator::Query> objects. These objects serves as the parents for the subquery. 

=back

=head2 Presentator::Condition

Condition is the interface providing informations about constraints within query.

=over 4

=item params()

returns array of C<Presentator::CndParam>.

=item param($name)

returns the parameter with name C<$name>.

=back

=head2 Presentator::Field

This interface provides informations about possible output fields.

=over 4

=item type()

returns the type of the output. It may be e.g. integer, float, string, date, options, variant. These may be used for formatting in future.

=item encoding()

returns the encoding of the field in data-source. It may be used for encoding conversions.

=back

=head2 Presentator::CndParam

provides informations about parameters that may be passed to the constraints (C<Presentator::Condition>).

=over 4

=item type()

returns the type of the parameter. It may be used e.g. for parameters validation or quoting.

=item classes()

returns the array reference (or undef) containing classes for C<Presentator::SecureParms>. It defines the possible sources for parameter binding. See also L<Presentator::SecureParms>.

=item default()

returns the default value provided for the parameter.

=back

=head1 SEE ALSO

L<Presentator::QueriesFactory>, L<Presentator::Select>

=head1 AUTHOR

Copyright (c) 2002 Igor Chladil (ichladil@quick.cz). All rights reserved.

This program is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
