package Presentator::SQLField;

use strict;

use vars qw/$VERSION/;

BEGIN {
	$VERSION='0.01';
}

1;

sub new {
	my ($proto, %attr) = @_;
	my $class = ref $proto || $proto;

	my @needed = qw/name encoding type/;

	my $self = {};

	map { warn("No parameter $_") unless $attr{$_}; } @needed;
	map {$self->{$_} = $attr{$_}} keys %attr;

	bless $self, $class;
}

sub setValue {
	$_[0]->{value} = $_[1];
}

sub value {
	$_[0]->{value};
}

sub encoding {
	$_[0]->{encoding};
}

sub type {
	$_[0]->{type};
}

sub name {
	$_[0]->{name};
}

__END__

=head1 NAME

Presentator::SQLField - fields returned from SQLResults

=head1 SYNOPSIS

	use Presentator::SQLField;

	my $field = Presentator::SQLField->new(
		name => 'id',
		type => 'integer',
		encoding => 'UTF-8',
		value => undef,
	);

	$field->setValue(193);
	$field->value(); #returns 193
	$field->name(); #returns 'id'
	$field->encoding(); #returns 'UTF-8'
	$field->type(); #returns 'integer'

=head1 INTERFACE

Functions C<value>, C<name>, C<encoding>, C<type> returns the actual values of current selected row from given field from SQLResults.

=head1 AUTHOR

Copyright (c) 2002 Igor Chladil (ichladil@quick.cz). All rights reserved.

This program is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
